@@ -15,7 +15,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Copyright 2012 Red Hat, Inc.
+ * Copyright 2012 - 2014 Red Hat, Inc.
  */
 
 #include "config.h"
@@ -24,10 +24,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include <nm-setting-connection.h>
-#include <nm-setting-bridge.h>
-#include <nm-utils.h>
-
 #include "page-bridge.h"
 #include "nm-connection-editor.h"
 #include "connection-helpers.h"
@@ -42,6 +38,7 @@ typedef struct {
 	GtkWindow *toplevel;
 
 	GtkSpinButton *ageing_time;
+	GtkCheckButton *mcast_snoop;
 	GtkCheckButton *stp;
 	GtkSpinButton *priority;
 	GtkSpinButton *forward_delay;
@@ -59,6 +56,7 @@ bridge_private_init (CEPageBridge *self)
 	builder = CE_PAGE (self)->builder;
 
 	priv->ageing_time = GTK_SPIN_BUTTON (gtk_builder_get_object (builder, "bridge_ageing_time"));
+	priv->mcast_snoop = GTK_CHECK_BUTTON (gtk_builder_get_object (builder, "bridge_mcast_snoop_checkbox"));
 	priv->stp = GTK_CHECK_BUTTON (gtk_builder_get_object (builder, "bridge_stp_checkbox"));
 	priv->priority = GTK_SPIN_BUTTON (gtk_builder_get_object (builder, "bridge_priority"));
 	priv->forward_delay = GTK_SPIN_BUTTON (gtk_builder_get_object (builder, "bridge_forward_delay"));
@@ -100,7 +98,7 @@ populate_ui (CEPageBridge *self)
 {
 	CEPageBridgePrivate *priv = CE_PAGE_BRIDGE_GET_PRIVATE (self);
 	NMSettingBridge *s_bridge = priv->setting;
-	gboolean stp;
+	gboolean stp, mcast_snoop;
 	int priority, forward_delay, hello_time, max_age;
 	int ageing_time;
 
@@ -111,6 +109,11 @@ populate_ui (CEPageBridge *self)
 	                  G_CALLBACK (stuff_changed),
 	                  self);
 
+	/* Multicast snooping */
+	mcast_snoop = nm_setting_bridge_get_multicast_snooping (s_bridge);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->mcast_snoop), mcast_snoop);
+	g_signal_connect (priv->mcast_snoop, "toggled", G_CALLBACK (stuff_changed), self);
+
 	/* STP */
 	g_signal_connect (priv->stp, "toggled",
 	                  G_CALLBACK (stp_toggled),
@@ -173,7 +176,7 @@ add_slave (CEPageMaster *master, NewConnectionResultFunc result_func)
 	CEPageBridgePrivate *priv = CE_PAGE_BRIDGE_GET_PRIVATE (self);
 
 	new_connection_dialog (priv->toplevel,
-	                       CE_PAGE (self)->settings,
+	                       CE_PAGE (self)->client,
 	                       connection_type_filter,
 	                       result_func,
 	                       master);
@@ -193,7 +196,6 @@ ce_page_bridge_new (NMConnectionEditor *editor,
                     NMConnection *connection,
                     GtkWindow *parent_window,
                     NMClient *client,
-                    NMRemoteSettings *settings,
                     const char **out_secrets_setting_name,
                     GError **error)
 {
@@ -205,7 +207,6 @@ ce_page_bridge_new (NMConnectionEditor *editor,
 	                                  connection,
 	                                  parent_window,
 	                                  client,
-	                                  settings,
 	                                  UIDIR "/ce-page-bridge.ui",
 	                                  "BridgePage",
 	                                  _("Bridge")));
@@ -234,12 +235,14 @@ ui_to_setting (CEPageBridge *self)
 {
 	CEPageBridgePrivate *priv = CE_PAGE_BRIDGE_GET_PRIVATE (self);
 	int ageing_time, priority, forward_delay, hello_time, max_age;
-	gboolean stp;
+	gboolean stp, mcast_snoop;
 
 	ageing_time = gtk_spin_button_get_value_as_int (priv->ageing_time);
+	mcast_snoop = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->mcast_snoop));
 	stp = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->stp));
 	g_object_set (G_OBJECT (priv->setting),
 	              NM_SETTING_BRIDGE_AGEING_TIME, ageing_time,
+	              NM_SETTING_BRIDGE_MULTICAST_SNOOPING, mcast_snoop,
 	              NM_SETTING_BRIDGE_STP, stp,
 	              NULL);
 
@@ -268,7 +271,7 @@ ce_page_validate_v (CEPage *page, NMConnection *connection, GError **error)
 		return FALSE;
 
 	ui_to_setting (self);
-	return nm_setting_verify (NM_SETTING (priv->setting), NULL, error);
+	return nm_setting_verify (NM_SETTING (priv->setting), connection, error);
 }
 
 static void
@@ -293,37 +296,34 @@ ce_page_bridge_class_init (CEPageBridgeClass *bridge_class)
 
 void
 bridge_connection_new (GtkWindow *parent,
-                     const char *detail,
-                     NMRemoteSettings *settings,
-                     PageNewConnectionResultFunc result_func,
-                     gpointer user_data)
+                       const char *detail,
+                       NMClient *client,
+                       PageNewConnectionResultFunc result_func,
+                       gpointer user_data)
 {
 	NMConnection *connection;
-	int bridge_num = 0, num;
-	GSList *connections, *iter;
+	NMSettingConnection *s_con;
+	int bridge_num = 0, num, i;
+	const GPtrArray *connections;
 	NMConnection *conn2;
-	NMSettingBridge *s_bridge;
 	const char *iface;
 	char *my_iface;
 
 	connection = ce_page_new_connection (_("Bridge connection %d"),
 	                                     NM_SETTING_BRIDGE_SETTING_NAME,
 	                                     TRUE,
-	                                     settings,
+	                                     client,
 	                                     user_data);
 	nm_connection_add_setting (connection, nm_setting_bridge_new ());
 
 	/* Find an available interface name */
-	connections = nm_remote_settings_list_connections (settings);
-	for (iter = connections; iter; iter = iter->next) {
-		conn2 = iter->data;
+	connections = nm_client_get_connections (client);
+	for (i = 0; i < connections->len; i++) {
+		conn2 = connections->pdata[i];
 
 		if (!nm_connection_is_type (conn2, NM_SETTING_BRIDGE_SETTING_NAME))
 			continue;
-		s_bridge = nm_connection_get_setting_bridge (conn2);
-		if (!s_bridge)
-			continue;
-		iface = nm_setting_bridge_get_interface_name (s_bridge);
+		iface = nm_connection_get_interface_name (conn2);
 		if (!iface || strncmp (iface, "bridge", 6) != 0 || !g_ascii_isdigit (iface[6]))
 			continue;
 
@@ -331,12 +331,11 @@ bridge_connection_new (GtkWindow *parent,
 		if (bridge_num <= num)
 			bridge_num = num + 1;
 	}
-	g_slist_free (connections);
 
+	s_con = nm_connection_get_setting_connection (connection);
 	my_iface = g_strdup_printf ("bridge%d", bridge_num);
-	s_bridge = nm_connection_get_setting_bridge (connection);
-	g_object_set (G_OBJECT (s_bridge),
-	              NM_SETTING_BRIDGE_INTERFACE_NAME, my_iface,
+	g_object_set (G_OBJECT (s_con),
+	              NM_SETTING_CONNECTION_INTERFACE_NAME, my_iface,
 	              NULL);
 	g_free (my_iface);
 
