@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 - 2011 Red Hat, Inc.
+ * Copyright 2008 - 2014 Red Hat, Inc.
  */
 
 #include "config.h"
@@ -28,11 +28,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include <nm-setting-connection.h>
-#include <nm-setting-wired.h>
-#include <nm-device-ethernet.h>
-#include <nm-utils.h>
-
 #include "page-ethernet.h"
 
 G_DEFINE_TYPE (CEPageEthernet, ce_page_ethernet, CE_TYPE_PAGE)
@@ -49,6 +44,9 @@ typedef struct {
 	GtkToggleButton *duplex;
 	GtkToggleButton *autonegotiate;
 	GtkSpinButton *mtu;
+	GtkToggleButton *wol_default, *wol_phy, *wol_unicast, *wol_multicast,
+	                *wol_broadcast, *wol_arp, *wol_magic;
+	GtkEntry *wol_passwd;
 } CEPageEthernetPrivate;
 
 #define PORT_DEFAULT  0
@@ -68,7 +66,7 @@ ethernet_private_init (CEPageEthernet *self)
 {
 	CEPageEthernetPrivate *priv = CE_PAGE_ETHERNET_GET_PRIVATE (self);
 	GtkBuilder *builder;
-	GtkWidget *align;
+	GtkWidget *vbox;
 	GtkLabel *label;
 
 	builder = CE_PAGE (self)->builder;
@@ -80,8 +78,9 @@ ethernet_private_init (CEPageEthernet *self)
 	                               "either by its interface name or permanent MAC or both. Examples: "
 	                               "\"em1\", \"3C:97:0E:42:1A:19\", \"em1 (3C:97:0E:42:1A:19)\""));
 
-	align = GTK_WIDGET (gtk_builder_get_object (builder, "ethernet_device_alignment"));
-	gtk_container_add (GTK_CONTAINER (align), GTK_WIDGET (priv->device_combo));
+	vbox = GTK_WIDGET (gtk_builder_get_object (builder, "ethernet_device_vbox"));
+	gtk_container_add (GTK_CONTAINER (vbox), GTK_WIDGET (priv->device_combo));
+	gtk_widget_set_halign (GTK_WIDGET (priv->device_combo), GTK_ALIGN_FILL);
 	gtk_widget_show_all (GTK_WIDGET (priv->device_combo));
 
 	/* Set mnemonic widget for Device label */
@@ -94,6 +93,14 @@ ethernet_private_init (CEPageEthernet *self)
 	priv->duplex = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "ethernet_duplex"));
 	priv->autonegotiate = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "ethernet_autonegotiate"));
 	priv->mtu = GTK_SPIN_BUTTON (gtk_builder_get_object (builder, "ethernet_mtu"));
+	priv->wol_default = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "wol_default"));
+	priv->wol_phy = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "wol_phy"));
+	priv->wol_unicast = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "wol_unicast"));
+	priv->wol_multicast = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "wol_multicast"));
+	priv->wol_broadcast = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "wol_broadcast"));
+	priv->wol_arp = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "wol_arp"));
+	priv->wol_magic = GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "wol_magic"));
+	priv->wol_passwd = GTK_ENTRY (gtk_builder_get_object (builder, "ethernet_wol_passwd"));
 }
 
 static void
@@ -102,6 +109,43 @@ stuff_changed (GtkWidget *w, gpointer user_data)
 	ce_page_changed (CE_PAGE (user_data));
 }
 
+static void
+wol_default_toggled_cb (GtkWidget *widget, gpointer user_data)
+{
+	CEPageEthernet *self = CE_PAGE_ETHERNET (user_data);
+	CEPageEthernetPrivate *priv = CE_PAGE_ETHERNET_GET_PRIVATE (self);
+	gboolean enabled, enabled_passwd;
+
+	enabled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->wol_phy), !enabled);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->wol_unicast), !enabled);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->wol_multicast), !enabled);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->wol_broadcast), !enabled);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->wol_arp), !enabled);
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->wol_magic), !enabled);
+
+	enabled_passwd = !enabled && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->wol_magic));
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->wol_passwd), enabled_passwd);
+
+	stuff_changed (NULL, self);
+}
+
+static void
+wol_magic_toggled_cb (GtkWidget *widget, gpointer user_data)
+{
+	CEPageEthernet *self = CE_PAGE_ETHERNET (user_data);
+	CEPageEthernetPrivate *priv = CE_PAGE_ETHERNET_GET_PRIVATE (self);
+	gboolean enabled;
+
+	enabled =    gtk_widget_get_sensitive (widget)
+	          && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->wol_passwd), enabled);
+
+	stuff_changed (NULL, self);
+}
+
 static void
 populate_ui (CEPageEthernet *self)
 {
@@ -112,8 +156,8 @@ populate_ui (CEPageEthernet *self)
 	int port_idx = PORT_DEFAULT;
 	int speed_idx;
 	int mtu_def;
-	const GByteArray *s_mac;
-	const char *s_ifname;
+	const char *s_mac, *s_ifname, *s_wol_passwd;
+	NMSettingWiredWakeOnLan wol;
 
 	/* Port */
 	port = nm_setting_wired_get_port (setting);
@@ -161,16 +205,17 @@ populate_ui (CEPageEthernet *self)
 	                              nm_setting_wired_get_auto_negotiate (setting));
 
 	/* Device ifname/MAC */
-	s_ifname = nm_connection_get_interface_name (CE_PAGE (self)->connection);
+        s_ifname = nm_connection_get_interface_name (CE_PAGE (self)->connection);
 	s_mac = nm_setting_wired_get_mac_address (setting);
 	ce_page_setup_device_combo (CE_PAGE (self), GTK_COMBO_BOX (priv->device_combo),
 	                            NM_TYPE_DEVICE_ETHERNET, s_ifname,
-	                            s_mac, ARPHRD_ETHER, NM_DEVICE_ETHERNET_PERMANENT_HW_ADDRESS, TRUE);
+	                            s_mac, NM_DEVICE_ETHERNET_PERMANENT_HW_ADDRESS, TRUE);
 	g_signal_connect (priv->device_combo, "changed", G_CALLBACK (stuff_changed), self);
 
 	/* Cloned MAC address */
 	s_mac = nm_setting_wired_get_cloned_mac_address (setting);
-	ce_page_mac_to_entry (s_mac, ARPHRD_ETHER, priv->cloned_mac);
+	if (s_mac)
+		gtk_entry_set_text (priv->cloned_mac, s_mac);
 	g_signal_connect (priv->cloned_mac, "changed", G_CALLBACK (stuff_changed), self);
 
 	/* MTU */
@@ -180,6 +225,31 @@ populate_ui (CEPageEthernet *self)
 	                  GINT_TO_POINTER (mtu_def));
 
 	gtk_spin_button_set_value (priv->mtu, (gdouble) nm_setting_wired_get_mtu (setting));
+
+	/* Wake-on-LAN */
+	wol = nm_setting_wired_get_wake_on_lan (priv->setting);
+	if (wol == NM_SETTING_WIRED_WAKE_ON_LAN_DEFAULT)
+		gtk_toggle_button_set_active (priv->wol_default, TRUE);
+	else {
+		if (wol & NM_SETTING_WIRED_WAKE_ON_LAN_PHY)
+			gtk_toggle_button_set_active (priv->wol_phy, TRUE);
+		if (wol & NM_SETTING_WIRED_WAKE_ON_LAN_UNICAST)
+			gtk_toggle_button_set_active (priv->wol_unicast, TRUE);
+		if (wol & NM_SETTING_WIRED_WAKE_ON_LAN_MULTICAST)
+			gtk_toggle_button_set_active (priv->wol_multicast, TRUE);
+		if (wol & NM_SETTING_WIRED_WAKE_ON_LAN_BROADCAST)
+			gtk_toggle_button_set_active (priv->wol_broadcast, TRUE);
+		if (wol & NM_SETTING_WIRED_WAKE_ON_LAN_ARP)
+			gtk_toggle_button_set_active (priv->wol_arp, TRUE);
+		if (wol & NM_SETTING_WIRED_WAKE_ON_LAN_MAGIC)
+			gtk_toggle_button_set_active (priv->wol_magic, TRUE);
+	}
+
+	/* Wake-on LAN password */
+	s_wol_passwd = nm_setting_wired_get_wake_on_lan_password (setting);
+	if (s_wol_passwd)
+		gtk_entry_set_text (priv->wol_passwd, s_wol_passwd);
+	g_signal_connect (priv->wol_passwd, "changed", G_CALLBACK (stuff_changed), self);
 }
 
 static void
@@ -200,6 +270,16 @@ finish_setup (CEPageEthernet *self, gpointer unused, GError *error, gpointer use
 	g_signal_connect (priv->autonegotiate, "toggled", G_CALLBACK (stuff_changed), self);
 	g_signal_connect (priv->mtu, "value-changed", G_CALLBACK (stuff_changed), self);
 
+	g_signal_connect (priv->wol_default,   "toggled", G_CALLBACK (wol_default_toggled_cb), self);
+	wol_default_toggled_cb (GTK_WIDGET (priv->wol_default), self);
+	g_signal_connect (priv->wol_phy,       "toggled", G_CALLBACK (stuff_changed), self);
+	g_signal_connect (priv->wol_unicast,   "toggled", G_CALLBACK (stuff_changed), self);
+	g_signal_connect (priv->wol_multicast, "toggled", G_CALLBACK (stuff_changed), self);
+	g_signal_connect (priv->wol_broadcast, "toggled", G_CALLBACK (stuff_changed), self);
+	g_signal_connect (priv->wol_arp,       "toggled", G_CALLBACK (stuff_changed), self);
+	g_signal_connect (priv->wol_magic,     "toggled", G_CALLBACK (wol_magic_toggled_cb), self);
+	wol_magic_toggled_cb (GTK_WIDGET (priv->wol_magic), self);
+
 	/* Hide widgets we don't yet support */
 	widget = GTK_WIDGET (gtk_builder_get_object (parent->builder, "ethernet_port_label"));
 	gtk_widget_hide (widget);
@@ -222,7 +302,6 @@ ce_page_ethernet_new (NMConnectionEditor *editor,
                       NMConnection *connection,
                       GtkWindow *parent_window,
                       NMClient *client,
-                      NMRemoteSettings *settings,
                       const char **out_secrets_setting_name,
                       GError **error)
 {
@@ -234,7 +313,6 @@ ce_page_ethernet_new (NMConnectionEditor *editor,
 	                                      connection,
 	                                      parent_window,
 	                                      client,
-	                                      settings,
 	                                      UIDIR "/ce-page-ethernet.ui",
 	                                      "EthernetPage",
 	                                      _("Ethernet")));
@@ -265,9 +343,11 @@ ui_to_setting (CEPageEthernet *self)
 	const char *port;
 	guint32 speed;
 	char *ifname = NULL;
-	GByteArray *device_mac = NULL;
-	GByteArray *cloned_mac = NULL;
+	char *device_mac = NULL;
+	const char *cloned_mac;
 	GtkWidget *entry;
+	NMSettingWiredWakeOnLan wol = NM_SETTING_WIRED_WAKE_ON_LAN_NONE;
+	const char *wol_passwd = NULL;
 
 	s_con = nm_connection_get_setting_connection (CE_PAGE (self)->connection);
 	g_return_if_fail (s_con != NULL);
@@ -313,27 +393,46 @@ ui_to_setting (CEPageEthernet *self)
 	entry = gtk_bin_get_child (GTK_BIN (priv->device_combo));
 	if (entry)
 		ce_page_device_entry_get (GTK_ENTRY (entry), ARPHRD_ETHER, TRUE, &ifname, &device_mac, NULL, NULL);
-	cloned_mac = ce_page_entry_to_mac (priv->cloned_mac, ARPHRD_ETHER, NULL);
+	cloned_mac = gtk_entry_get_text (priv->cloned_mac);
+
+	/* Wake-on-LAN */
+	if (gtk_toggle_button_get_active (priv->wol_default))
+		wol = NM_SETTING_WIRED_WAKE_ON_LAN_DEFAULT;
+	else {
+		if (gtk_toggle_button_get_active (priv->wol_phy))
+			wol |= NM_SETTING_WIRED_WAKE_ON_LAN_PHY;
+		if (gtk_toggle_button_get_active (priv->wol_unicast))
+			wol |= NM_SETTING_WIRED_WAKE_ON_LAN_UNICAST;
+		if (gtk_toggle_button_get_active (priv->wol_multicast))
+			wol |= NM_SETTING_WIRED_WAKE_ON_LAN_MULTICAST;
+		if (gtk_toggle_button_get_active (priv->wol_broadcast))
+			wol |= NM_SETTING_WIRED_WAKE_ON_LAN_BROADCAST;
+		if (gtk_toggle_button_get_active (priv->wol_arp))
+			wol |= NM_SETTING_WIRED_WAKE_ON_LAN_ARP;
+		if (gtk_toggle_button_get_active (priv->wol_magic))
+			wol |= NM_SETTING_WIRED_WAKE_ON_LAN_MAGIC;
+	}
+
+	if (gtk_widget_get_sensitive (GTK_WIDGET (priv->wol_passwd)))
+		wol_passwd = gtk_entry_get_text (priv->wol_passwd);
 
 	g_object_set (s_con,
 	              NM_SETTING_CONNECTION_INTERFACE_NAME, ifname,
 	              NULL);
 	g_object_set (priv->setting,
 	              NM_SETTING_WIRED_MAC_ADDRESS, device_mac,
-	              NM_SETTING_WIRED_CLONED_MAC_ADDRESS, cloned_mac,
+	              NM_SETTING_WIRED_CLONED_MAC_ADDRESS, cloned_mac && *cloned_mac ? cloned_mac : NULL,
 	              NM_SETTING_WIRED_PORT, port,
 	              NM_SETTING_WIRED_SPEED, speed,
 	              NM_SETTING_WIRED_DUPLEX, gtk_toggle_button_get_active (priv->duplex) ? "full" : "half",
 	              NM_SETTING_WIRED_AUTO_NEGOTIATE, gtk_toggle_button_get_active (priv->autonegotiate),
 	              NM_SETTING_WIRED_MTU, (guint32) gtk_spin_button_get_value_as_int (priv->mtu),
+	              NM_SETTING_WIRED_WAKE_ON_LAN, wol,
+	              NM_SETTING_WIRED_WAKE_ON_LAN_PASSWORD, wol_passwd && *wol_passwd ? wol_passwd : NULL,
 	              NULL);
 
 	g_free (ifname);
-	if (device_mac)
-		g_byte_array_free (device_mac, TRUE);
-	if (cloned_mac)
-		g_byte_array_free (cloned_mac, TRUE);
-
+	g_free (device_mac);
 }
 
 static gboolean
@@ -352,6 +451,11 @@ ce_page_validate_v (CEPage *page, NMConnection *connection, GError **error)
 	if (!ce_page_mac_entry_valid (priv->cloned_mac, ARPHRD_ETHER, _("cloned MAC"), error))
 		return FALSE;
 
+	if (gtk_widget_get_sensitive (GTK_WIDGET (priv->wol_passwd))) {
+		if (!ce_page_mac_entry_valid (priv->wol_passwd, ARPHRD_ETHER, _("Wake-on-LAN password"), error))
+			return FALSE;
+	}
+
 	ui_to_setting (self);
 	return nm_setting_verify (NM_SETTING (priv->setting), NULL, error);
 }
@@ -377,7 +481,7 @@ ce_page_ethernet_class_init (CEPageEthernetClass *ethernet_class)
 void
 ethernet_connection_new (GtkWindow *parent,
                          const char *detail,
-                         NMRemoteSettings *settings,
+                         NMClient *client,
                          PageNewConnectionResultFunc result_func,
                          gpointer user_data)
 {
@@ -386,7 +490,7 @@ ethernet_connection_new (GtkWindow *parent,
 	connection = ce_page_new_connection (_("Ethernet connection %d"),
 	                                     NM_SETTING_WIRED_SETTING_NAME,
 	                                     TRUE,
-	                                     settings,
+	                                     client,
 	                                     user_data);
 	nm_connection_add_setting (connection, nm_setting_wired_new ());
 
