@@ -15,7 +15,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Copyright 2012 Red Hat, Inc.
+ * Copyright 2012 - 2014 Red Hat, Inc.
  */
 
 #include "config.h"
@@ -23,11 +23,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include <nm-setting-connection.h>
-#include <nm-setting-infiniband.h>
-#include <nm-device-infiniband.h>
-#include <nm-utils.h>
-
 #include <net/if_arp.h>
 #include <linux/if_infiniband.h>
 
@@ -54,7 +49,7 @@ infiniband_private_init (CEPageInfiniband *self)
 {
 	CEPageInfinibandPrivate *priv = CE_PAGE_INFINIBAND_GET_PRIVATE (self);
 	GtkBuilder *builder;
-	GtkWidget *align;
+	GtkWidget *vbox;
 	GtkLabel *label;
 
 	builder = CE_PAGE (self)->builder;
@@ -67,8 +62,9 @@ infiniband_private_init (CEPageInfiniband *self)
 	                               "\"ib0\", \"80:00:00:48:fe:80:00:00:00:00:00:00:00:02:c9:03:00:00:0f:65\", "
 	                               "\"ib0 (80:00:00:48:fe:80:00:00:00:00:00:00:00:02:c9:03:00:00:0f:65)\""));
 
-	align = GTK_WIDGET (gtk_builder_get_object (builder, "infiniband_device_alignment"));
-	gtk_container_add (GTK_CONTAINER (align), GTK_WIDGET (priv->device_combo));
+	vbox = GTK_WIDGET (gtk_builder_get_object (builder, "infiniband_device_vbox"));
+	gtk_container_add (GTK_CONTAINER (vbox), GTK_WIDGET (priv->device_combo));
+	gtk_widget_set_halign (GTK_WIDGET (priv->device_combo), GTK_ALIGN_FILL);
 	gtk_widget_show_all (GTK_WIDGET (priv->device_combo));
 
 	/* Set mnemonic widget for Device label */
@@ -93,8 +89,7 @@ populate_ui (CEPageInfiniband *self)
 	const char *mode;
 	int mode_idx = TRANSPORT_MODE_DATAGRAM;
 	int mtu_def;
-	const GByteArray *s_mac;
-	const char *s_ifname;
+	const char *s_ifname, *s_mac;
 
 	/* Port */
 	mode = nm_setting_infiniband_get_transport_mode (setting);
@@ -107,11 +102,11 @@ populate_ui (CEPageInfiniband *self)
 	gtk_combo_box_set_active (priv->transport_mode, mode_idx);
 
 	/* Device */
-	s_ifname = nm_connection_get_interface_name (CE_PAGE (self)->connection);
+        s_ifname = nm_connection_get_interface_name (CE_PAGE (self)->connection);
 	s_mac = nm_setting_infiniband_get_mac_address (setting);
 	ce_page_setup_device_combo (CE_PAGE (self), GTK_COMBO_BOX (priv->device_combo),
 	                            NM_TYPE_DEVICE_INFINIBAND, s_ifname,
-	                            s_mac, ARPHRD_INFINIBAND, NM_DEVICE_INFINIBAND_HW_ADDRESS, TRUE);
+	                            s_mac, NM_DEVICE_INFINIBAND_HW_ADDRESS, TRUE);
 	g_signal_connect (priv->device_combo, "changed", G_CALLBACK (stuff_changed), self);
 
 	/* MTU */
@@ -142,7 +137,6 @@ ce_page_infiniband_new (NMConnectionEditor *editor,
                         NMConnection *connection,
                         GtkWindow *parent_window,
                         NMClient *client,
-                        NMRemoteSettings *settings,
                         const char **out_secrets_setting_name,
                         GError **error)
 {
@@ -154,7 +148,6 @@ ce_page_infiniband_new (NMConnectionEditor *editor,
 	                                        connection,
 	                                        parent_window,
 	                                        client,
-	                                        settings,
 	                                        UIDIR "/ce-page-infiniband.ui",
 	                                        "InfinibandPage",
 	                                        _("InfiniBand")));
@@ -185,7 +178,7 @@ ui_to_setting (CEPageInfiniband *self)
 	NMSettingConnection *s_con;
 	const char *mode;
 	char *ifname = NULL;
-	GByteArray *device_mac = NULL;
+	char *device_mac = NULL;
 	GtkWidget *entry;
 
 	s_con = nm_connection_get_setting_connection (CE_PAGE (self)->connection);
@@ -211,8 +204,7 @@ ui_to_setting (CEPageInfiniband *self)
 	              NULL);
 
 	g_free (ifname);
-	if (device_mac)
-		g_byte_array_free (device_mac, TRUE);
+	g_free (device_mac);
 }
 
 static gboolean
@@ -229,7 +221,7 @@ ce_page_validate_v (CEPage *page, NMConnection *connection, GError **error)
 	}
 
 	ui_to_setting (self);
-	return nm_setting_verify (NM_SETTING (priv->setting), NULL, error);
+	return nm_setting_verify (NM_SETTING (priv->setting), connection, error);
 }
 
 static void
@@ -253,7 +245,7 @@ ce_page_infiniband_class_init (CEPageInfinibandClass *infiniband_class)
 void
 infiniband_connection_new (GtkWindow *parent,
                            const char *detail,
-                           NMRemoteSettings *settings,
+                           NMClient *client,
                            PageNewConnectionResultFunc result_func,
                            gpointer user_data)
 {
@@ -262,7 +254,7 @@ infiniband_connection_new (GtkWindow *parent,
 	connection = ce_page_new_connection (_("InfiniBand connection %d"),
 	                                     NM_SETTING_INFINIBAND_SETTING_NAME,
 	                                     TRUE,
-	                                     settings,
+	                                     client,
 	                                     user_data);
 	nm_connection_add_setting (connection, nm_setting_infiniband_new ());
 
