@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2008 - 2012 Red Hat, Inc.
+ * Copyright 2008 - 2014 Red Hat, Inc.
  */
 
 #include "config.h"
@@ -33,16 +33,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include <nm-setting-connection.h>
-#include <nm-setting-ip4-config.h>
-#include <nm-setting-wired.h>
-#include <nm-setting-wireless.h>
-#include <nm-setting-gsm.h>
-#include <nm-setting-cdma.h>
-#include <nm-setting-pppoe.h>
-#include <nm-setting-vpn.h>
-#include <nm-utils.h>
-
 #include "page-ip4.h"
 #include "ip4-routes-dialog.h"
 #include "connection-helpers.h"
@@ -58,7 +48,7 @@ G_DEFINE_TYPE (CEPageIP4, ce_page_ip4, CE_TYPE_PAGE)
 #define COL_LAST COL_GATEWAY
 
 typedef struct {
-	NMSettingIP4Config *setting;
+	NMSettingIPConfig *setting;
 	char *connection_id;
 	GType connection_type;
 
@@ -133,7 +123,7 @@ ip4_private_init (CEPageIP4 *self, NMConnection *connection)
 	connection_type = nm_setting_connection_get_connection_type (s_con);
 	g_assert (connection_type);
 
-	priv->connection_type = nm_connection_lookup_setting_type (connection_type);
+	priv->connection_type = nm_setting_lookup_type (connection_type);
 
 	if (priv->connection_type == NM_TYPE_SETTING_VPN) {
 		str_auto = _("Automatic (VPN)");
@@ -359,7 +349,7 @@ static void
 populate_ui (CEPageIP4 *self)
 {
 	CEPageIP4Private *priv = CE_PAGE_IP4_GET_PRIVATE (self);
-	NMSettingIP4Config *setting = priv->setting;
+	NMSettingIPConfig *setting = priv->setting;
 	GtkListStore *store;
 	GtkTreeIter model_iter;
 	int method = IP4_METHOD_AUTO;
@@ -370,7 +360,7 @@ populate_ui (CEPageIP4 *self)
 
 	/* Method */
 	gtk_combo_box_set_active (priv->method, 0);
-	str_method = nm_setting_ip4_config_get_method (setting);
+	str_method = nm_setting_ip_config_get_method (setting);
 	if (str_method) {
 		if (!strcmp (str_method, NM_SETTING_IP4_CONFIG_METHOD_LINK_LOCAL))
 			method = IP4_METHOD_LINK_LOCAL;
@@ -382,7 +372,7 @@ populate_ui (CEPageIP4 *self)
 			method = IP4_METHOD_DISABLED;
 	}
 
-	if (method == IP4_METHOD_AUTO && nm_setting_ip4_config_get_ignore_auto_dns (setting))
+	if (method == IP4_METHOD_AUTO && nm_setting_ip_config_get_ignore_auto_dns (setting))
 		method = IP4_METHOD_AUTO_ADDRESSES;
 
 	info.method = method;
@@ -391,29 +381,24 @@ populate_ui (CEPageIP4 *self)
 
 	/* Addresses */
 	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
-	for (i = 0; i < nm_setting_ip4_config_get_num_addresses (setting); i++) {
-		NMIP4Address *addr = nm_setting_ip4_config_get_address (setting, i);
-		struct in_addr tmp_addr;
-		char buf[INET_ADDRSTRLEN + 1];
+	for (i = 0; i < nm_setting_ip_config_get_num_addresses (setting); i++) {
+		NMIPAddress *addr = nm_setting_ip_config_get_address (setting, i);
+		char buf[32];
 
 		if (!addr) {
 			g_warning ("%s: empty IP4 Address structure!", __func__);
 			continue;
 		}
 
+		snprintf (buf, sizeof (buf), "%u", nm_ip_address_get_prefix (addr));
+
 		gtk_list_store_append (store, &model_iter);
-
-		tmp_addr.s_addr = nm_ip4_address_get_address (addr);
-		(void) inet_ntop (AF_INET, &tmp_addr, &buf[0], sizeof (buf));
-		gtk_list_store_set (store, &model_iter, COL_ADDRESS, buf, -1);
-
-		tmp_addr.s_addr = nm_utils_ip4_prefix_to_netmask (nm_ip4_address_get_prefix (addr));
-		(void) inet_ntop (AF_INET, &tmp_addr, &buf[0], sizeof (buf));
-		gtk_list_store_set (store, &model_iter, COL_PREFIX, buf, -1);
-
-		tmp_addr.s_addr = nm_ip4_address_get_gateway (addr);
-		(void) inet_ntop (AF_INET, &tmp_addr, &buf[0], sizeof (buf));
-		gtk_list_store_set (store, &model_iter, COL_GATEWAY, buf, -1);
+		gtk_list_store_set (store, &model_iter,
+		                    COL_ADDRESS, nm_ip_address_get_address (addr),
+		                    COL_PREFIX, buf,
+		                    /* FIXME */
+		                    COL_GATEWAY, i == 0 ? nm_setting_ip_config_get_gateway (setting) : NULL,
+		                    -1);
 	}
 
 	gtk_tree_view_set_model (priv->addr_list, GTK_TREE_MODEL (store));
@@ -423,42 +408,40 @@ populate_ui (CEPageIP4 *self)
 
 	/* DNS servers */
 	string = g_string_new ("");
-	for (i = 0; i < nm_setting_ip4_config_get_num_dns (setting); i++) {
-		struct in_addr tmp_addr;
-		char buf[INET_ADDRSTRLEN + 1];
+	for (i = 0; i < nm_setting_ip_config_get_num_dns (setting); i++) {
+		const char *dns;
 
-		tmp_addr.s_addr = nm_setting_ip4_config_get_dns (setting, i);
-		if (!tmp_addr.s_addr)
+		dns = nm_setting_ip_config_get_dns (setting, i);
+		if (!dns)
 			continue;
 
-		(void) inet_ntop (AF_INET, &tmp_addr, &buf[0], sizeof (buf));
 		if (string->len)
 			g_string_append (string, ", ");
-		g_string_append (string, buf);
+		g_string_append (string, dns);
 	}
 	gtk_entry_set_text (priv->dns_servers, string->str);
 	g_string_free (string, TRUE);
 
 	/* DNS searches */
 	string = g_string_new ("");
-	for (i = 0; i < nm_setting_ip4_config_get_num_dns_searches (setting); i++) {
+	for (i = 0; i < nm_setting_ip_config_get_num_dns_searches (setting); i++) {
 		if (string->len)
 			g_string_append (string, ", ");
-		g_string_append (string, nm_setting_ip4_config_get_dns_search (setting, i));
+		g_string_append (string, nm_setting_ip_config_get_dns_search (setting, i));
 	}
 	gtk_entry_set_text (priv->dns_searches, string->str);
 	g_string_free (string, TRUE);
 
 	if ((method == IP4_METHOD_AUTO) || (method == IP4_METHOD_AUTO_ADDRESSES)) {
-		if (nm_setting_ip4_config_get_dhcp_client_id (setting)) {
+		if (nm_setting_ip4_config_get_dhcp_client_id (NM_SETTING_IP4_CONFIG (setting))) {
 			gtk_entry_set_text (priv->dhcp_client_id,
-			                    nm_setting_ip4_config_get_dhcp_client_id (setting));
+			                    nm_setting_ip4_config_get_dhcp_client_id (NM_SETTING_IP4_CONFIG (setting)));
 		}
 	}
 
 	/* IPv4 required */
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->ip4_required),
-	                              !nm_setting_ip4_config_get_may_fail (setting));
+	                              !nm_setting_ip_config_get_may_fail (setting));
 }
 
 static void
@@ -572,6 +555,9 @@ cell_editing_canceled (GtkCellRenderer *renderer, gpointer user_data)
 	priv->last_column = -1;
 }
 
+#define DO_NOT_CYCLE_TAG "do-not-cycle"
+#define DIRECTION_TAG    "direction"
+
 static void
 cell_edited (GtkCellRendererText *cell,
              const gchar *path_string,
@@ -585,6 +571,9 @@ cell_edited (GtkCellRendererText *cell,
 	GtkTreeIter iter;
 	guint32 column;
 	GtkTreeViewColumn *next_col;
+	GtkCellRenderer *next_cell;
+	gboolean can_cycle;
+	int direction, tmp;
 
 	/* Free auxiliary stuff */
 	g_free (priv->last_edited);
@@ -618,10 +607,22 @@ cell_edited (GtkCellRendererText *cell,
 		g_free (prefix);
 	}
 
-	/* Move focus to the next column */
-	column = (column >= COL_LAST) ? 0 : column + 1;
+	/* Move focus to the next/previous column */
+	can_cycle = g_object_get_data (G_OBJECT (cell), DO_NOT_CYCLE_TAG) == NULL;
+	direction = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), DIRECTION_TAG));
+	g_object_set_data (G_OBJECT (cell), DIRECTION_TAG, NULL);
+	g_object_set_data (G_OBJECT (cell), DO_NOT_CYCLE_TAG, NULL);
+	if (direction == 0)  /* Move forward by default */
+		direction = 1;
+
+	tmp = column + direction;
+	if (can_cycle)
+		column = tmp < 0 ? COL_LAST : tmp > COL_LAST ? 0 : tmp;
+	else
+		column = tmp;
 	next_col = gtk_tree_view_get_column (priv->addr_list, column);
-	gtk_tree_view_set_cursor_on_cell (priv->addr_list, path, next_col, priv->addr_cells[column], TRUE);
+	next_cell = column <= COL_LAST ? priv->addr_cells[column] : NULL;
+	gtk_tree_view_set_cursor_on_cell (priv->addr_list, path, next_col, next_cell, TRUE);
 
 	gtk_tree_path_free (path);
 	ce_page_changed (CE_PAGE (self));
@@ -692,12 +693,15 @@ parse_netmask (const char *str, guint32 *prefix)
 	struct in_addr tmp_addr;
 	glong tmp_prefix;
 
+	if (!str || !*str)
+		return FALSE;
+
 	errno = 0;
 
 	/* Is it a prefix? */
 	if (!strchr (str, '.')) {
 		tmp_prefix = strtol (str, NULL, 10);
-		if (!errno && tmp_prefix >= 0 && tmp_prefix <= 32) {
+		if (!errno && tmp_prefix > 0 && tmp_prefix <= 32) {
 			*prefix = tmp_prefix;
 			return TRUE;
 		}
@@ -712,12 +716,63 @@ parse_netmask (const char *str, guint32 *prefix)
 	return FALSE;
 }
 
+static gboolean
+is_address_unspecified (const char *str)
+{
+	struct in_addr addr;
+
+	if (!str)
+		return FALSE;
+
+	return (   inet_pton (AF_INET, str, &addr) == 1
+	        && addr.s_addr == INADDR_ANY);
+}
+
+static gboolean
+gateway_matches_address (const char *gw_str, const char *addr_str, guint32 prefix)
+{
+	struct in_addr gw, addr;
+	guint32 netmask;
+
+	if (!gw_str || inet_pton (AF_INET, gw_str, &gw) != 1)
+		return FALSE;
+	if (!addr_str || inet_pton (AF_INET, addr_str, &addr) != 1)
+		return FALSE;
+
+	netmask = nm_utils_ip4_prefix_to_netmask (prefix);
+	return ((addr.s_addr & netmask) == (gw.s_addr & netmask));
+}
+
+static gboolean
+possibly_wrong_gateway (GtkTreeModel *model, GtkTreeIter *iter, const char *gw_str)
+{
+	char *addr_str, *prefix_str;
+	gboolean addr_valid;
+	guint32 prefix;
+
+	gtk_tree_model_get (model, iter, COL_ADDRESS, &addr_str, -1);
+	gtk_tree_model_get (model, iter, COL_PREFIX, &prefix_str, -1);
+	addr_valid =   addr_str && *addr_str && nm_utils_ipaddr_valid (AF_INET, addr_str) && !is_address_unspecified (addr_str)
+	            && parse_netmask (prefix_str, &prefix);
+
+	if (addr_valid && !gateway_matches_address (gw_str, addr_str, prefix))
+		return TRUE;
+	else
+		return FALSE;
+}
+
+typedef struct {
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	guint column;
+} AddressLineInfo;
+
 static gboolean
 cell_changed_cb (GtkEditable *editable,
                  gpointer user_data)
 {
+	AddressLineInfo *info = (AddressLineInfo *) user_data;
 	char *cell_text;
-	guint column;
 	GdkRGBA rgba;
 	gboolean value_valid = FALSE;
 	const char *colorname = NULL;
@@ -725,8 +780,7 @@ cell_changed_cb (GtkEditable *editable,
 	cell_text = gtk_editable_get_chars (editable, 0, -1);
 
 	/* The COL_PREFIX can contain IP address or prefix */
-	column = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (user_data), "column"));
-	if (column == COL_PREFIX) {
+	if (info->column == COL_PREFIX) {
 		guint32 tmp_prefix;
 
 		value_valid = parse_netmask (cell_text, &tmp_prefix);
@@ -735,45 +789,80 @@ cell_changed_cb (GtkEditable *editable,
 
 		if (inet_pton (AF_INET, cell_text, &tmp_addr) > 0)
 			value_valid = TRUE;
+
+		 /* 0.0.0.0 is not accepted for address */
+		if (info->column == COL_ADDRESS && tmp_addr.s_addr == 0)
+			value_valid = FALSE;
+		/* Consider empty gateway as valid */
+		if (!*cell_text && info->column == COL_GATEWAY)
+			value_valid = TRUE;
 	}
 
 	/* Change cell's background color while editing */
 	colorname = value_valid ? "lightgreen" : "red";
 
+	/* Check gateway against address and prefix */
+	if (   info->column == COL_GATEWAY
+	    && value_valid
+	    && possibly_wrong_gateway (info->model, &info->iter, cell_text))
+		colorname = "yellow";
+
 	gdk_rgba_parse (&rgba, colorname);
-	gtk_widget_override_background_color (GTK_WIDGET (editable), GTK_STATE_FLAG_NORMAL, &rgba);
+	utils_override_bg_color (GTK_WIDGET (editable), &rgba);
 
 	g_free (cell_text);
 	return FALSE;
 }
 
 static gboolean
-key_pressed_cb (GtkWidget *widget,
-                GdkEvent *event,
-                gpointer user_data)
+key_pressed_cb (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 {
-	GdkKeymapKey *keys = NULL;
-	gint n_keys;
+	GdkModifierType modifiers;
+	GtkCellRenderer *cell = (GtkCellRenderer *) user_data;
+
+	modifiers = event->state & gtk_accelerator_get_default_mod_mask ();
 
 	/*
-	 * Tab should behave the same way as Enter (cycling on cells).
+	 * Change some keys so that they work properly:
+	 * We want:
+	 *   - Tab should behave the same way as Enter (cycling on cells),
+	 *   - Shift-Tab should move in backwards direction.
+	 *   - Down arrow moves as Enter, but we have to handle Down arrow on
+	 *     key pad.
+	 *   - Up arrow should move backwards and we also have to handle Up arrow
+	 *     on key pad.
+	 *   - Enter should end editing when pressed on last column.
 	 *
-	 * Previously, we had finished cell editing, which appeared to work:
-	 *   gtk_cell_editable_editing_done (GTK_CELL_EDITABLE (widget));
-	 * But unfortunately, it showed up crash occurred with XIM input (GTK_IM_MODULE=xim).
+	 * Note: gtk_cell_editable_editing_done (GTK_CELL_EDITABLE (widget)) cannot be called
+	 * in this function, because it would crash with XIM input (GTK_IM_MODULE=xim), see
 	 * https://bugzilla.redhat.com/show_bug.cgi?id=747368
 	 */
-	if (event->type == GDK_KEY_PRESS && event->key.keyval == GDK_KEY_Tab) {
-		/* Get hardware keycode for GDK_KEY_Return */
-		if (gdk_keymap_get_entries_for_keyval (gdk_keymap_get_default (), GDK_KEY_Return, &keys, &n_keys)) {
-			/* Change 'Tab' to 'Enter' key */
-			event->key.keyval = GDK_KEY_Return;
-			event->key.hardware_keycode = keys[0].keycode;
-		}
-		g_free (keys);
-	}
 
-	return FALSE;
+	if (event->keyval == GDK_KEY_Tab && modifiers == 0) {
+		/* Tab */
+		g_object_set_data (G_OBJECT (cell), DIRECTION_TAG, GINT_TO_POINTER (1));
+		utils_fake_return_key (event);
+	} else if (event->keyval == GDK_KEY_ISO_Left_Tab && modifiers == GDK_SHIFT_MASK) {
+		/* Shift-Tab */
+		g_object_set_data (G_OBJECT (cell), DIRECTION_TAG, GINT_TO_POINTER (-1));
+		utils_fake_return_key (event);
+	} else if (event->keyval == GDK_KEY_KP_Down)
+		event->keyval = GDK_KEY_Down;
+	else if (event->keyval == GDK_KEY_Up || event->keyval == GDK_KEY_KP_Up) {
+		event->keyval = GDK_KEY_Up;
+		g_object_set_data (G_OBJECT (cell), DIRECTION_TAG, GINT_TO_POINTER (-1));
+	} else if (   event->keyval == GDK_KEY_Return
+	           || event->keyval == GDK_KEY_ISO_Enter
+	           || event->keyval == GDK_KEY_KP_Enter)
+		g_object_set_data (G_OBJECT (cell), DO_NOT_CYCLE_TAG, GUINT_TO_POINTER (TRUE));
+
+	return FALSE; /* Allow default handler to be called */
+}
+
+static void
+address_line_info_destroy (AddressLineInfo *info)
+{
+	g_slice_free (AddressLineInfo, info);
 }
 
 static void
@@ -784,6 +873,9 @@ cell_editing_started (GtkCellRenderer *cell,
 {
 	CEPageIP4 *self = CE_PAGE_IP4 (user_data);
 	CEPageIP4Private *priv = CE_PAGE_IP4_GET_PRIVATE (self);
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	AddressLineInfo *info;
 
 	if (!GTK_IS_ENTRY (editable)) {
 		g_warning ("%s: Unexpected cell editable type.", __func__);
@@ -807,14 +899,21 @@ cell_editing_started (GtkCellRenderer *cell,
 	                        user_data);
 
 	/* Set up handler for IP verifying and changing cell background */
-	g_signal_connect (G_OBJECT (editable), "changed",
-	                  (GCallback) cell_changed_cb,
-	                  cell);
+	model = gtk_tree_view_get_model (priv->addr_list);
+	gtk_tree_model_get_iter_from_string (model, &iter, priv->last_path);
+	info = g_slice_new0 (AddressLineInfo);
+	info->model = model;
+	info->iter = iter;
+	info->column = priv->last_column;
+	g_signal_connect_data (G_OBJECT (editable), "changed",
+	                       (GCallback) cell_changed_cb,
+	                       info,
+	                       (GClosureNotify) address_line_info_destroy, 0);
 
 	/* Set up key pressed handler - need to handle Tab key */
 	g_signal_connect (G_OBJECT (editable), "key-press-event",
 	                  (GCallback) key_pressed_cb,
-	                  user_data);
+	                  cell);
 }
 
 static void
@@ -850,7 +949,7 @@ routes_button_clicked_cb (GtkWidget *button, gpointer user_data)
 	toplevel = gtk_widget_get_toplevel (CE_PAGE (self)->page);
 	g_return_if_fail (gtk_widget_is_toplevel (toplevel));
 
-	method = nm_setting_ip4_config_get_method (priv->setting);
+	method = nm_setting_ip_config_get_method (priv->setting);
 	if (!method || !strcmp (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO))
 		automatic = TRUE;
 
@@ -911,6 +1010,41 @@ tree_view_button_pressed_cb (GtkWidget *widget,
 	return FALSE;
 }
 
+static void
+cell_error_data_func (GtkTreeViewColumn *tree_column,
+                      GtkCellRenderer *cell,
+                      GtkTreeModel *tree_model,
+                      GtkTreeIter *iter,
+                      gpointer data)
+{
+	guint32 col = GPOINTER_TO_UINT (data);
+	char *value = NULL;
+	const char *color = NULL;
+	guint32 prefix;
+	gboolean invalid = FALSE;
+
+	gtk_tree_model_get (tree_model, iter, col, &value, -1);
+
+	if (col == COL_ADDRESS)
+		invalid =    !value || !*value || !nm_utils_ipaddr_valid (AF_INET, value)
+		          || is_address_unspecified (value);
+	else if (col == COL_PREFIX)
+		invalid = !parse_netmask (value, &prefix);
+	else if (col == COL_GATEWAY) {
+		invalid = value && *value && !nm_utils_ipaddr_valid (AF_INET, value);
+
+		/* Check gateway against address and prefix */
+		if (!invalid && possibly_wrong_gateway (tree_model, iter, value))
+			color = "#DDC000"; /* darker than "yellow", else selected text is hard to read */
+	} else
+		g_warn_if_reached ();
+
+	if (invalid)
+		color = "red";
+	utils_set_cell_background (cell, color, color ? value : NULL);
+	g_free (value);
+}
+
 static void
 finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_data)
 {
@@ -941,6 +1075,8 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 	column = gtk_tree_view_get_column (GTK_TREE_VIEW (priv->addr_list), offset - 1);
 	gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), TRUE);
 	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);
+	gtk_tree_view_column_set_cell_data_func (column, renderer, cell_error_data_func,
+	                                         GUINT_TO_POINTER (COL_ADDRESS), NULL);
 
 	/* Prefix/netmask column */
 	renderer = gtk_cell_renderer_text_new ();
@@ -958,6 +1094,8 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 	column = gtk_tree_view_get_column (GTK_TREE_VIEW (priv->addr_list), offset - 1);
 	gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), TRUE);
 	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);
+	gtk_tree_view_column_set_cell_data_func (column, renderer, cell_error_data_func,
+	                                         GUINT_TO_POINTER (COL_PREFIX), NULL);
 
 	/* Gateway column */
 	renderer = gtk_cell_renderer_text_new ();
@@ -975,6 +1113,8 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 	column = gtk_tree_view_get_column (GTK_TREE_VIEW (priv->addr_list), offset - 1);
 	gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), TRUE);
 	gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);
+	gtk_tree_view_column_set_cell_data_func (column, renderer, cell_error_data_func,
+	                                         GUINT_TO_POINTER (COL_GATEWAY), NULL);
 
 	g_signal_connect (priv->addr_list, "button-press-event", G_CALLBACK (tree_view_button_pressed_cb), self);
 
@@ -1005,7 +1145,6 @@ ce_page_ip4_new (NMConnectionEditor *editor,
                  NMConnection *connection,
                  GtkWindow *parent_window,
                  NMClient *client,
-                 NMRemoteSettings *settings,
                  const char **out_secrets_setting_name,
                  GError **error)
 {
@@ -1018,7 +1157,6 @@ ce_page_ip4_new (NMConnectionEditor *editor,
 	                                 connection,
 	                                 parent_window,
 	                                 client,
-	                                 settings,
 	                                 UIDIR "/ce-page-ip4.ui",
 	                                 "IP4Page",
 	                                 _("IPv4 Settings")));
@@ -1037,10 +1175,7 @@ ce_page_ip4_new (NMConnectionEditor *editor,
 	priv->connection_id = g_strdup (nm_setting_connection_get_id (s_con));
 
 	priv->setting = nm_connection_get_setting_ip4_config (connection);
-	if (!priv->setting) {
-		priv->setting = NM_SETTING_IP4_CONFIG (nm_setting_ip4_config_new ());
-		nm_connection_add_setting (connection, NM_SETTING (priv->setting));
-	}
+	g_assert (priv->setting);
 
 	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
 
@@ -1050,7 +1185,7 @@ ce_page_ip4_new (NMConnectionEditor *editor,
 static void
 free_one_addr (gpointer data)
 {
-	g_array_free ((GArray *) data, TRUE);
+	nm_ip_address_unref ((NMIPAddress *) data);
 }
 
 static gboolean
@@ -1061,9 +1196,11 @@ ui_to_setting (CEPageIP4 *self, GError **error)
 	GtkTreeIter tree_iter;
 	int int_method = IP4_METHOD_AUTO;
 	const char *method;
-	GArray *dns_servers = NULL;
-	GSList *search_domains = NULL;
+	GPtrArray *tmp_array = NULL;
+	char **dns_servers = NULL;
+	char **search_domains = NULL;
 	GPtrArray *addresses = NULL;
+	char *gateway = NULL;
 	gboolean valid = FALSE, iter_valid;
 	const char *text;
 	gboolean ignore_auto_dns = FALSE;
@@ -1104,49 +1241,54 @@ ui_to_setting (CEPageIP4 *self, GError **error)
 
 	addresses = g_ptr_array_sized_new (1);
 	while (iter_valid) {
-		char *item = NULL;
-		struct in_addr tmp_addr, tmp_gateway = { 0 };
-		GArray *addr;
-		guint32 empty_val = 0, prefix;
+		char *addr = NULL, *netmask = NULL, *addr_gw = NULL;
+		NMIPAddress *nm_addr;
+		guint32 prefix;
 
-		gtk_tree_model_get (model, &tree_iter, COL_ADDRESS, &item, -1);
-		if (!item || inet_pton (AF_INET, item, &tmp_addr) <= 0) {
-			g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv4 address \"%s\" invalid"), item ? item : "");
-			g_free (item);
-			goto out;
-		}
-		g_free (item);
+		gtk_tree_model_get (model, &tree_iter,
+		                    COL_ADDRESS, &addr,
+		                    COL_PREFIX, &netmask,
+		                    COL_GATEWAY, &addr_gw,
+		                    -1);
 
-		gtk_tree_model_get (model, &tree_iter, COL_PREFIX, &item, -1);
-		if (!item) {
-			g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv4 address netmask missing"));
+		if (   !addr
+		    || !nm_utils_ipaddr_valid (AF_INET, addr)
+		    || is_address_unspecified (addr)) {
+			g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv4 address \"%s\" invalid"), addr ? addr : "");
+			g_free (addr);
+			g_free (netmask);
+			g_free (addr_gw);
 			goto out;
 		}
 
-		if (!parse_netmask (item, &prefix)) {
-			g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv4 address netmask \"%s\" invalid"), item);
-			g_free (item);
+		if (!parse_netmask (netmask, &prefix)) {
+			g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv4 address netmask \"%s\" invalid"), netmask ? netmask : "");
+			g_free (addr);
+			g_free (netmask);
+			g_free (addr_gw);
 			goto out;
 		}
-		g_free (item);
 
 		/* Gateway is optional... */
-		gtk_tree_model_get (model, &tree_iter, COL_GATEWAY, &item, -1);
-		if (item && strlen (item) && inet_pton (AF_INET, item, &tmp_gateway) <= 0) {
-			g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv4 gateway \"%s\" invalid"), item);
-			g_free (item);
+		if (addr_gw && *addr_gw && !nm_utils_ipaddr_valid (AF_INET, addr_gw)) {
+			g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv4 gateway \"%s\" invalid"), addr_gw);
+			g_free (addr);
+			g_free (netmask);
+			g_free (addr_gw);
 			goto out;
 		}
-		g_free (item);
 
-		addr = g_array_sized_new (FALSE, TRUE, sizeof (guint32), 3);
-		g_array_append_val (addr, tmp_addr.s_addr);
-		g_array_append_val (addr, prefix);
-		if (tmp_gateway.s_addr)
-			g_array_append_val (addr, tmp_gateway.s_addr);
-		else
-			g_array_append_val (addr, empty_val);
-		g_ptr_array_add (addresses, addr);
+		nm_addr = nm_ip_address_new (AF_INET, addr, prefix, NULL);
+		g_ptr_array_add (addresses, nm_addr);
+
+		if (addresses->len == 1 && addr_gw && *addr_gw) {
+			gateway = addr_gw;
+			addr_gw = NULL;
+		}
+
+		g_free (addr);
+		g_free (netmask);
+		g_free (addr_gw);
 
 		iter_valid = gtk_tree_model_iter_next (model, &tree_iter);
 	}
@@ -1158,8 +1300,7 @@ ui_to_setting (CEPageIP4 *self, GError **error)
 	}
 
 	/* DNS servers */
-	dns_servers = g_array_new (FALSE, FALSE, sizeof (guint));
-
+	tmp_array = g_ptr_array_new ();
 	text = gtk_entry_get_text (GTK_ENTRY (priv->dns_servers));
 	if (text && strlen (text)) {
 		items = g_strsplit_set (text, ", ;:", 0);
@@ -1167,21 +1308,25 @@ ui_to_setting (CEPageIP4 *self, GError **error)
 			struct in_addr tmp_addr;
 			char *stripped = g_strstrip (*iter);
 
-			if (!strlen (stripped))
+			if (!*stripped)
 				continue;
 
 			if (inet_pton (AF_INET, stripped, &tmp_addr))
-				g_array_append_val (dns_servers, tmp_addr.s_addr);
+				g_ptr_array_add (tmp_array, g_strdup (stripped));
 			else {
 				g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC, _("IPv4 DNS server \"%s\" invalid"), stripped);
 				g_strfreev (items);
+				g_ptr_array_free (tmp_array, TRUE);
 				goto out;
 			}
 		}
 		g_strfreev (items);
 	}
+	g_ptr_array_add (tmp_array, NULL);
+	dns_servers = (char **) g_ptr_array_free (tmp_array, FALSE);
 
 	/* Search domains */
+	tmp_array = g_ptr_array_new ();
 	text = gtk_entry_get_text (GTK_ENTRY (priv->dns_searches));
 	if (text && strlen (text)) {
 		items = g_strsplit_set (text, ", ;:", 0);
@@ -1189,12 +1334,12 @@ ui_to_setting (CEPageIP4 *self, GError **error)
 			char *stripped = g_strstrip (*iter);
 
 			if (strlen (stripped))
-				search_domains = g_slist_prepend (search_domains, g_strdup (stripped));
+				g_ptr_array_add (tmp_array, g_strdup (stripped));
 		}
 		g_strfreev (items);
 	}
-
-	search_domains = g_slist_reverse (search_domains);
+	g_ptr_array_add (tmp_array, NULL);
+	search_domains = (char **) g_ptr_array_free (tmp_array, FALSE);
 
 	/* DHCP client ID */
 	if (!strcmp (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO)) {
@@ -1207,13 +1352,14 @@ ui_to_setting (CEPageIP4 *self, GError **error)
 
 	/* Update setting */
 	g_object_set (priv->setting,
-	              NM_SETTING_IP4_CONFIG_METHOD, method,
-	              NM_SETTING_IP4_CONFIG_ADDRESSES, addresses,
-	              NM_SETTING_IP4_CONFIG_DNS, dns_servers,
-	              NM_SETTING_IP4_CONFIG_DNS_SEARCH, search_domains,
-	              NM_SETTING_IP4_CONFIG_IGNORE_AUTO_DNS, ignore_auto_dns,
+	              NM_SETTING_IP_CONFIG_METHOD, method,
+	              NM_SETTING_IP_CONFIG_ADDRESSES, addresses,
+	              NM_SETTING_IP_CONFIG_GATEWAY, gateway,
+	              NM_SETTING_IP_CONFIG_DNS, dns_servers,
+	              NM_SETTING_IP_CONFIG_DNS_SEARCH, search_domains,
+	              NM_SETTING_IP_CONFIG_IGNORE_AUTO_DNS, ignore_auto_dns,
 	              NM_SETTING_IP4_CONFIG_DHCP_CLIENT_ID, dhcp_client_id,
-	              NM_SETTING_IP4_CONFIG_MAY_FAIL, may_fail,
+	              NM_SETTING_IP_CONFIG_MAY_FAIL, may_fail,
 	              NULL);
 	valid = TRUE;
 
@@ -1222,12 +1368,10 @@ out:
 		g_ptr_array_foreach (addresses, (GFunc) free_one_addr, NULL);
 		g_ptr_array_free (addresses, TRUE);
 	}
+	g_free (gateway);
 
-	if (dns_servers)
-		g_array_free (dns_servers, TRUE);
-
-	g_slist_foreach (search_domains, (GFunc) g_free, NULL);
-	g_slist_free (search_domains);
+	g_strfreev (dns_servers);
+	g_strfreev (search_domains);
 
 	return valid;
 }
@@ -1300,7 +1444,7 @@ change_method_combo (CEPage *page, gboolean is_hotspot)
 	if (is_hotspot) {
 		if (priv->hotspot_method_idx == -1) {
 			int method = IP4_METHOD_SHARED;
-			if (g_strcmp0 (nm_setting_ip4_config_get_method (priv->setting),
+			if (g_strcmp0 (nm_setting_ip_config_get_method (priv->setting),
 			               NM_SETTING_IP4_CONFIG_METHOD_DISABLED) == 0)
 				method = IP4_METHOD_DISABLED;
 			gtk_combo_box_set_active (priv->method, method);
