@@ -38,30 +38,7 @@
 
 static GMainLoop *loop = NULL;
 gboolean shell_debug = FALSE;
-
-static void
-signal_handler (int signo, siginfo_t *info, void *data)
-{
-	if (signo == SIGINT || signo == SIGTERM) {
-		g_message ("PID %d (we are %d) sent signal %d, shutting down...",
-		           info->si_pid, getpid (), signo);
-		g_main_loop_quit (loop);
-	}
-}
-
-static void
-setup_signals (void)
-{
-	struct sigaction action;
-	sigset_t mask;
-
-	sigemptyset (&mask);
-	action.sa_sigaction = signal_handler;
-	action.sa_mask = mask;
-	action.sa_flags = SA_SIGINFO;
-	sigaction (SIGTERM,  &action, NULL);
-	sigaction (SIGINT,  &action, NULL);
-}
+gboolean with_agent = TRUE;
 
 static void
 usage (const char *progname)
@@ -89,6 +66,8 @@ int main (int argc, char *argv[])
 		}
 		if (!strcmp (argv[i], "--shell-debug"))
 			shell_debug = TRUE;
+		if (!strcmp (argv[i], "--no-agent"))
+			with_agent = FALSE;
 	}
 
 	bindtextdomain (GETTEXT_PACKAGE, NMALOCALEDIR);
@@ -102,7 +81,6 @@ int main (int argc, char *argv[])
 	if (applet == NULL)
 		exit (1);
 
-	setup_signals ();
 	g_main_loop_run (loop);
 
 	g_object_unref (G_OBJECT (applet));
