@@ -36,14 +36,15 @@ G_DEFINE_TYPE (NMMbMenuItem, nm_mb_menu_item, GTK_TYPE_IMAGE_MENU_ITEM);
 #define NM_MB_MENU_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_MB_MENU_ITEM, NMMbMenuItemPrivate))
 
 typedef struct {
-	GtkWidget *desc;
-	char *desc_string;
+#ifndef ENABLE_INDICATOR
 	GtkWidget *strength;
-	guint32    int_strength;
 	GtkWidget *detail;
 	GtkWidget *hbox;
+	GtkWidget *desc;
+#endif
 
-	gboolean   destroyed;
+	char *desc_string;
+	guint32    int_strength;
 } NMMbMenuItemPrivate;
 
 static const char *
@@ -70,14 +71,30 @@ get_tech_name (guint32 tech)
 		return _("HSPA");
 	case MB_TECH_HSPA_PLUS:
 		return _("HSPA+");
-	case MB_TECH_WIMAX:
-		return _("WiMAX");
 	case MB_TECH_LTE:
 		return _("LTE");
 	default:
-		break;
+		return NULL;
 	}
-	return NULL;
+}
+
+static void
+update_label (NMMbMenuItem *item, gboolean use_bold)
+{
+	NMMbMenuItemPrivate *priv = NM_MB_MENU_ITEM_GET_PRIVATE (item);
+
+#ifdef ENABLE_INDICATOR
+	gtk_menu_item_set_label (GTK_MENU_ITEM (item), priv->desc_string);
+#else
+	gtk_label_set_use_markup (GTK_LABEL (priv->desc), use_bold);
+	if (use_bold) {
+		char *markup = g_markup_printf_escaped ("<b>%s</b>", priv->desc_string);
+
+		gtk_label_set_markup (GTK_LABEL (priv->desc), markup);
+		g_free (markup);
+	} else
+		gtk_label_set_text (GTK_LABEL (priv->desc), priv->desc_string);
+#endif
 }
 
 GtkWidget *
@@ -92,18 +109,15 @@ nm_mb_menu_item_new (const char *connection_name,
 {
 	NMMbMenuItem *item;
 	NMMbMenuItemPrivate *priv;
-	const char *tech_name = NULL;
+	const char *tech_name;
 
 	item = g_object_new (NM_TYPE_MB_MENU_ITEM, NULL);
-	if (!item)
-		return NULL;
+	g_assert (item);
 
 	priv = NM_MB_MENU_ITEM_GET_PRIVATE (item);
 	priv->int_strength = strength;
 
-	/* WiMAX doesn't show tech name */
-	if (technology != MB_TECH_WIMAX)
-		tech_name = get_tech_name (technology);
+	tech_name = get_tech_name (technology);
 
 	/* Construct the description string */
 	switch (state) {
@@ -170,24 +184,26 @@ nm_mb_menu_item_new (const char *connection_name,
 		break;
 	}
 
-	if (enabled && connection_name && active) {
-		char *markup;
-
-		gtk_label_set_use_markup (GTK_LABEL (priv->desc), TRUE);
-		markup = g_markup_printf_escaped ("<b>%s</b>", priv->desc_string);
-		gtk_label_set_markup (GTK_LABEL (priv->desc), markup);
-		g_free (markup);
-	} else {
-		/* Disconnected and disabled states */
-		gtk_label_set_use_markup (GTK_LABEL (priv->desc), FALSE);
-		gtk_label_set_text (GTK_LABEL (priv->desc), priv->desc_string);
-	}
+	update_label (item, (enabled && connection_name && active));
 
 	/* And the strength icon, if we have strength information at all */
 	if (enabled && strength) {
-		GdkPixbuf *pixbuf = nma_icon_check_and_load (mobile_helper_get_quality_icon_name (strength), applet);
+		const char *icon_name = mobile_helper_get_quality_icon_name (strength);
+		GdkPixbuf *pixbuf = nma_icon_check_and_load (icon_name, applet);
 
+#ifdef ENABLE_INDICATOR
+#ifdef DBUSMENU_PIXMAP_SUPPORT
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), gtk_image_new_from_pixbuf (pixbuf));
+#else
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
+		                               gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU));
+		pixbuf = NULL;
+#endif
+		/* For some reason we must always re-set always-show after setting the image */
+		gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (item), TRUE);
+#else
 		gtk_image_set_from_pixbuf (GTK_IMAGE (priv->strength), pixbuf);
+#endif
 	}
 
 	return GTK_WIDGET (item);
@@ -198,6 +214,7 @@ nm_mb_menu_item_new (const char *connection_name,
 static void
 nm_mb_menu_item_init (NMMbMenuItem *self)
 {
+#ifndef ENABLE_INDICATOR
 	NMMbMenuItemPrivate *priv = NM_MB_MENU_ITEM_GET_PRIVATE (self);
 
 	priv->hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
@@ -213,26 +230,17 @@ nm_mb_menu_item_init (NMMbMenuItem *self)
 	gtk_widget_show (priv->desc);
 	gtk_widget_show (priv->strength);
 	gtk_widget_show (priv->hbox);
+#else
+	gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (self), TRUE);
+#endif
 }
 
 static void
-dispose (GObject *object)
+finalize (GObject *object)
 {
-	NMMbMenuItem *self = NM_MB_MENU_ITEM (object);
-	NMMbMenuItemPrivate *priv = NM_MB_MENU_ITEM_GET_PRIVATE (self);
+	g_free (NM_MB_MENU_ITEM_GET_PRIVATE (object)->desc_string);
 
-	if (priv->destroyed) {
-		G_OBJECT_CLASS (nm_mb_menu_item_parent_class)->dispose (object);
-		return;
-	}
-	priv->destroyed = TRUE;
-
-	gtk_widget_destroy (priv->desc);
-	gtk_widget_destroy (priv->strength);
-	gtk_widget_destroy (priv->hbox);
-	g_free (priv->desc_string);
-
-	G_OBJECT_CLASS (nm_mb_menu_item_parent_class)->dispose (object);
+	G_OBJECT_CLASS (nm_mb_menu_item_parent_class)->finalize (object);
 }
 
 static void
@@ -243,6 +251,6 @@ nm_mb_menu_item_class_init (NMMbMenuItemClass *klass)
 	g_type_class_add_private (klass, sizeof (NMMbMenuItemPrivate));
 
 	/* virtual methods */
-	object_class->dispose = dispose;
+	object_class->finalize = finalize;
 }
 
